PROGRAM BIGBANG;

(*
 BIGBANG - A program to create universe files for VGA Planets.

 Copyright 1999 by Frederik Arbeiter	<Frederik.Arbeiter@gmx.net>
 This program if free software as defined in the General Public License (GPL)
 published by the Free Software Foundation (FSF)
*)

USES
 crt, fdesk, param,
 graph, graphx,
 strings, files, math;

CONST

VERSION			='0.21';
MAXREC			=16;

MAXRAD			=990;
AVRX			=1997;
AVRY			=2000;
MAGIC			=998681; { don't touch!}

STAT_ANY	=0;
STAT_HW		=1;
STAT_MOON	=2;

GEO_DISK	=1;
GEO_SPOKES	=2;
GEO_SPIRALS	=3;

DENSITY_CONST	=1;
DENSITY_LINEAR	=2;

NAMES_ORIGINAL	=1;
NAMES_EXPLICIT	=2;
NAMES_SECRECY	=3;

COL_PLAYER : array[0..11] of byte = (lightblue,
                                     white, yellow,lightgreen,
									 white, yellow, lightgreen,
									 white, yellow, lightgreen,
									 white, yellow);
COL_HW			= red;
COL_MOON		= magenta;

TYPE

planetType = record
    posX, posY		:integer;
    name			:string[20];
    player			:integer;
    status			:integer;
    end;

galaxyType =array[1..500] of planetType;

configType = record
    name_project					:string[8];
    describe_project				:string;
    num_sectors						:integer;
    name_sector						:array[1..11] of string[10];
    num_vortex_pl, num_moons		:integer;
    r_max, r_vortex, r_hw, r_moon	:integer;
    type_geo, type_density			:integer;
    spiral_slope					:real;
    spoking_focus					:real;
    var_r, var_phi					:real;
    m_var_r, m_var_phi				:real;
    naming_policy					:integer;
    hw_mindist						:integer;
    end;

saveRecordType = record
	versid		:string[24];
    note		:string[60];
    source		:configType;
    galaxy		:galaxyType;
	end;

VAR
 galaxy		:galaxyType;
 config		:configType;
 _debug_	:boolean;
 _verbose_	:boolean;

procedure defaults;
var
 i	:integer;
begin
config.name_project := 'std';
if(posParam('/PN') <> 0) then
	config.name_project := getParam('/PN');
config.describe_project := 'No description.';
config.num_sectors := 11;
config.num_vortex_pl := 50;
config.num_moons := 6;
for i := 1 to 11 do
	config.name_sector[i] := chr(ord('A')+i-1);
config.r_max := 990;
config.r_vortex := 200;
config.r_hw := 600;
config.r_moon := 49;
config.type_geo := GEO_DISK;
config.spiral_slope := 0.65;
config.spoking_focus := 0.5;
config.type_density := DENSITY_CONST;
config.var_r := 0;
config.var_phi := 0;
config.m_var_r := 0;
config.m_var_phi := 0;
config.naming_policy := NAMES_EXPLICIT;
config.hw_mindist := 50;

for i := 1 to 500 do
	begin
    galaxy[i].name := 'planet-' + strOfInt(i);
    galaxy[i].posX := AVRX;
    galaxy[i].posY := AVRY;
    galaxy[i].player := 0;
    galaxy[i].status := STAT_ANY;
    end;
end;

procedure configure;

	procedure project;
    begin
	cls(fdesk.colorset.bc);
	headline(lightgray, yellow, 'BIGBANG - Universe configuration : Project name');
    gotoXY(2, 3);
    config.name_project := strInput('Project Name [8 chars]', config.name_project, 8, true  );
    gotoXY(2, 5);
    config.describe_project := strInput('Project description [255 chars]', config.describe_project, 255, true);
    gotoXY(2, 7);
    config.naming_policy := select('Naming policy', 'Original'#255'names Explicit'#255'HW'#255'names Hidden'#255'HWs');
    end;

    procedure sectors;
    var
     i	:integer;
    begin
	cls(fdesk.colorset.bc);
	headline(lightgray, yellow, 'BIGBANG - Universe configuration : Sector number & names');
    gotoXY(2, 3);
    config.num_sectors := intInput('Number of sectors [1..11]', config.num_sectors, 4, true);
    if((config.num_sectors > 11) or (config.num_sectors < 1)) then
		config.num_sectors := 11;
    for i := 1 to config.num_sectors do
    	config.name_sector[i] := strInput('Sector #'+strOfInt(i)+'''s name [10 chars]', config.name_sector[i], 10, true);
    end;

    procedure dimensions;
    begin
	cls(fdesk.colorset.bc);
	headline(lightgray, yellow, 'BIGBANG - Universe configuration : Sizes');
    gotoXY(2, 3);
    config.r_max := intInput('Cluster radius [200..995]', config.r_max, 4, true);
    config.r_vortex := intInput('Inner vortex radius [10..500]', config.r_vortex, 4, true);
    config.r_hw := intInput('Homeworld radius [100..900]', config.r_hw, 4, true);
    config.r_moon := intInput('Moonsystems radius around HW [10..100]', config.r_moon, 4, true);
    config.hw_mindist := intInput('Minimum Distance between HWs [0..250]', config.hw_mindist, 4, true);
    end;

    procedure distribution;
    begin
    cls(fdesk.colorset.bc);
	headline(lightgray, yellow, 'BIGBANG - Universe configuration : Density and geometry');
    gotoXY(1, 3);
    config.num_vortex_pl := intInput('Number of planets in vortex zone [10..200]', config.num_vortex_pl, 4, true);
    config.num_moons := intInput('Number of moon planets [0..20]', config.num_moons, 4, true);
    config.var_r := realInput('"Hidefactor" (radial) for HWs[0..1]', config.var_r, 4, true);
    config.var_phi :=realInput('"Hidefactor" (tangetial) for HWs [0..1]', config.var_phi, 4, true);
    config.m_var_r := realInput('"Hidefactor" (radial) for moons [0..1]', config.m_var_r, 4, true);
    config.m_var_phi :=realInput('"Hidefactor" (tangetial) for moons [0..1]', config.m_var_phi, 4, true);
    config.type_geo := select('Geometry type', 'Disk Spokes Spiral');
    if(config.type_geo = GEO_SPIRALS) then
        config.spiral_slope := realInput('Spiral slope [0..5] (real)', config.spiral_slope, 4, true);
    if(config.type_geo = GEO_SPIRALS) or (config.type_geo = GEO_SPOKES) then
        config.spoking_focus := realInput('"Spoking" focus [0..8] (real)', config.spoking_focus, 2, true);
    config.type_density := select('Radial density d(r) type', 'Constant Linear');
    end;

    procedure save;
    var
     f	:file of configType;
    begin
    cls(fdesk.colorset.bc);
	headline(lightgray, yellow, 'BIGBANG - Universe configuration : Save universe parameters');
    gotoXY(2, 3);
    if(choice('Really write configuration to '+upStr(config.name_project+'.unc'))) then
    	begin
        assign(f,config.name_project+'.unc');
        rewrite(f);
        write(f, config);
        close(f);
        end;
    writeLn(' OK!');
    writeLn;
    waitMessage('Press any key to continue');
    end;

    procedure load;
    var
     f		:file of configType;
     name   :string[8];
     path	:string;
    begin
    cls(fdesk.colorset.bc);
	headline(lightgray, yellow, 'BIGBANG - Universe configuration : Load existing config file (.unc)');
    statusline(lightgray, black, '');
    gotoXY(2, 3);
    name := strInput('Project name', config.name_project, 8, true);
	writeLn;
	path := upStr(name+'.unc');
	if(not fileExist(path)) then
		begin
    	writeLn(' The file ', path, ' doesn''t exist in the current directory!');
    	writeLn;
    	waitMessage('Press any key to continue');
    	exit;
    	end;
    writeLn(' Trying to load .\'+path);
    assign(f, path);
    {$I-}
    reset(f);
    {$I+}
    if(IOresult <> 0) then
    	begin
        writeLn(' Failed to open !');
        waitMessage('Press any key to continue');
        exit;
        end;
	read(f, config);
    writeLn(' OK.');
    close(f);
    end;

var
 i	:integer;
begin
	repeat
	cls(fdesk.colorset.bc);
	headline(lightgray, yellow, 'BIGBANG - Universe configuration');
    statusline(lightgray, black, '');
 	with config do
	begin
    colors(fdesk.colorset.bc, yellow);
    writeLn;writeLn;writeLn('Current configuration');writeLn;
    write(' Project name : ', name_project);
    write('.   Naming policy : ');
    case naming_policy of
     NAMES_ORIGINAL:write(' Create no names.');
     NAMES_EXPLICIT:write(' Identify HWs and moons');
     NAMES_SECRECY: write(' Use sectorname and id# as name');
    end;
    writeLn;
    writeLn(' Sectors : ', num_sectors, ' sectors named  [', name_sector[1], ' ... ', name_sector[num_sectors], ']' );
    writeLn(' Outer radius = ', r_max, '  HW radius = ', r_hw,
	        '  Vortex radius = ', r_vortex, '  Moon''s radius = ', r_moon);
    write(' Geometry type : ');
    	case type_geo of
         GEO_DISK:write('DISK');
         GEO_SPOKES:write('SPOKES , focus = ', spoking_focus:0:3);
         GEO_SPIRALS:write('SPIRALS, focus = ', spoking_focus:0:3, ' spiral slope = ', spiral_slope:0:3);
        end;
    write('   Density d(r) type : ');
    	case type_density of
         DENSITY_CONST: write('CONSTANT');
         DENSITY_LINEAR: write('LINEAR');
        end;
    writeLn;
    writeLn(' HW   "hide factor" : radial = ', var_r:0:3, ' tangential = ', var_phi:0:3);
    writeLn(' Moon "hide factor" : radial = ', m_var_r:0:3, ' tangential = ', m_var_phi:0:3);
    writeLn;
    gotoXY(3, getY);
	i := menu('Config', 'Project Sectors Dimensions Star'#255'Distribution Save'#255'Config Load'#255'Config Leave'#255'Menu');
    	case i of
        1:project;
        2:sectors;
        3:dimensions;
        4:distribution;
        5:save;
        6:load;
    	end;
	end; {with config}
	until i=7;
end;

procedure create(mute: boolean);

	function existStar(idx, x, y:integer):integer;
	var
	 rval	:integer;
 	 i		:integer;
	begin
	rval := 0;
	for i:=1 to 500 do
 		if(x = galaxy[i].posX) and (y = galaxy[i].posY) then
    		if(i <> idx) then
				begin
                rval := i;
                break;
	            end;
	existStar := rval;
	end;

	procedure randxy(ri, ra:integer; phiL, phiH:real; var x, y:integer);

		function density(ri, ra, r:integer; phiL, phiH, phi:real):integer; {return value ranges 0..1000}
	    var
        a_r, a_phi  :real;
	     rval		:integer;
	    begin

        	case config.type_density of
            DENSITY_CONST:a_r := r/ra;
            DENSITY_LINEAR:a_r := 1;
        	end;

            case config.type_geo of
            GEO_DISK:a_phi := 1;
            GEO_SPOKES, GEO_SPIRALS:
				if(r>config.r_vortex) then
					a_phi := pot(0.5 * (1+sin(-2*pi/config.num_sectors + phi*config.num_sectors)), config.spoking_focus)
                else
                	a_phi := 1;
            end;

	    rval := round(a_r*a_phi*1000);
	    density := rval;
	    end;


	var
	 r		:integer;
	 phi    :real;

	begin
		repeat
			repeat
			r := ri + random(ra - ri);
			phi := phiL+random(round((phiH-phiL)*10000))/10000;
			until random(1000) < density(ri, ra, r, phiL, phiH, phi);

        if(config.type_geo = GEO_SPIRALS) then
			phi := phi + 2*pi*config.spiral_slope*r/config.r_max;

		x := round(cos(phi)*r)+AVRX;
		y := round(sin(phi)*r)+AVRY;
		until true; { not existStar(x, y);}
	end;

	procedure swapPlanets(var p1, p2:planetType);
	var
	 buf	:planetType;
	begin
	buf := p1;
	p1 := p2;
	p2 := buf;
	end;


   function test_hw_dist : boolean;
   var
    hwid		:array[1..11] of integer;
    i, j		:integer;
    rval		:boolean;
    d, dx, dy		:double;
   begin
   j := 0;
   for i:=1 to 500 do
   		if(galaxy[i].status = STAT_HW) then
        	begin
            inc(j);
            hwid[j] := i;
            end;
   rval := true;
   for i:=1 to config.num_sectors do
      for j:=1 to config.num_sectors do
      	begin
        if(i = j) then
        	continue;
        dx := galaxy[hwid[i]].posX-galaxy[hwid[j]].posX;
        dy := galaxy[hwid[i]].posY-galaxy[hwid[j]].posY;
        d := sqrt(sqr(dx) + sqr(dy));
        if(config.hw_mindist > d) then
        	begin
            rval := false;
            if(_debug_) then
            	waitMessage(' small distance #'+strOfInt(hwid[i])+' to #'+ strOfInt(hwid[j])+ ' is only '+strOf(d, 2));
            end;
        end;
   test_hw_dist := rval;
   end;

var
 i, j, idx		:integer;
 sps, ansiv		:integer;
 sec			:integer;
 phi, secwidth, moonwidth	:real;
 hwx, hwy, r	:integer;
 count			:array[0..11] of integer;
 countm			:array[1..11] of integer;
 f				:file of planetType;
 chksum, diff	:longint;
 clean			:boolean;
 frustration	:integer;
 c				:char;

begin
cls(fdesk.colorset.bc);
textColor(green);
writeLn;
if(not mute) then
	writeLn('Creating a new universe :');
sps := (500 - config.num_vortex_pl) div config.num_sectors; (* stars per sector *)
ansiv := 500 - config.num_sectors * sps;	(* actual number of stars in vortex *)
idx := 0;
frustration := 0;

repeat

textColor(white);
if(not mute) then
	writeLn(' creating ', ansiv, ' stars in the deadly vortex ...');
for i:=1 to ansiv do
	begin
    inc(idx);
    randXY(0, config.r_vortex, 0, 2*pi,
           galaxy[i].posX, galaxy[i].posY);
    galaxy[i].status := STAT_ANY;
    galaxy[i].player := 0;
    end;
if(not mute) then
	writeLn(' creating ',config.num_sectors,' sectors around the vortex ...');
secwidth := 2*pi/config.num_sectors;
moonwidth := 2*pi/config.num_moons;
for sec := 1 to config.num_sectors do
	begin
	if(_verbose_ or _debug_) and (not mute) then
    	begin
		writeLn('    creating the ',config.name_sector[sec],' sector ...');
    	writeLn('        ... it''s homeworld with ', config.num_moons, ' moons');
    	end;
    inc(idx);
    phi := (sec-0.5)*secwidth + secwidth*config.var_phi*2*(random -0.5);
    r := round(config.r_hw + config.r_hw*config.var_r*2*(random -0.5));
    if(config.type_geo = GEO_SPIRALS) then
		phi := phi + 2*pi*config.spiral_slope*r/config.r_max;

    hwx := round(cos(phi) * r)+AVRX;
    hwy := round(sin(phi) * r)+AVRY;
    galaxy[idx].posX := hwx;
    galaxy[idx].posY := hwy;
    galaxy[idx].player := sec;
    galaxy[idx].status := STAT_HW;
    for i:=1 to config.num_moons do
    	begin
        inc(idx);
    	phi := (i-0.5)*moonwidth + 2*pi*config.m_var_phi*2*(random - 0.5);
   		r := round(config.r_moon + config.r_moon*config.m_var_r*2*(random - 0.5));
    	galaxy[idx].posX := round(cos(phi) * r)+hwx;
    	galaxy[idx].posY := round(sin(phi) * r)+hwy;
        galaxy[idx].player := sec;
        galaxy[idx].status := STAT_MOON;
        end;
    if(_verbose_ or _debug_) and (not mute) then
		writeLn('        ... and additionally ', sps-config.num_moons-1, ' stars ');
    for i:= 1 to sps-config.num_moons-1 do
    	begin
        inc(idx);
        randxy(config.r_vortex, config.r_max, (sec-1)*2*pi/config.num_sectors, sec*2*pi/config.num_sectors,
		       galaxy[idx].posX, galaxy[idx].posY);
        galaxy[idx].player := sec;
        galaxy[idx].status := STAT_ANY;
        end;
    end;

if(not mute) then
	begin
	writeLn;
	writeLn('Now adjusting to to magic number ...');
    end;

    j := 0;
    repeat
    inc(j);
    if(j > 100) then
    	begin
        textColor(lightred);
        writeLn;
        writeLn(#7'OOOPS! You stumbled upon a big bad bug in BIGBANG!');
        writeLn('I could not adjust to the magic number after 100 iterations.');
        writeLn;
        writeLn('If you want to help improving BIGBANG send an eMail to the');
        writeLn('author stating: BIGBANG couldn''t adjust to magic number.');
        writeLn('Critical diff = ', diff);
        writeLn;
        textColor(yellow);
        writeLn('Meanwhile, to continue creating a new universe please do as');
        writeLn('if nothing happened and start the universe generator again.');
        writeLn('DO NOT COMPILE OR SAVE THE CURRENT UNIVERSE DATA! It will not');
        writeLn('work with VGA planets due to a bad checksum.');
        writeLn;
        textColor(white);
        writeLn(' Press <B> to continue');
        repeat c := readkey until c in ['B', 'b'];
        exit;
        end;
    chksum := 0;
    for i:=1 to 500 do
    	inc(chksum, galaxy[i].posX);
    diff := MAGIC - chksum;
    if(_debug_) and (not mute) then
		writeLn('debug: magic diff=',diff);
	if(diff >= 500) then
    	for i:=1 to 500 do
        	inc(galaxy[i].posX, 1);
    if(diff <= -500) then
    	for i:=1 to 500 do
        	dec(galaxy[i].posX, 1);
    if(diff < 500) and (diff > 0) then
    	for i:= 1 to ansiv do
            begin
        	inc(galaxy[i].posX);
            if(abs(i) = abs(diff)) then
            	break;
            end;
    if(diff > -500) and (diff < 0) then
    	for i:= 1 to ansiv do
        	begin
        	dec(galaxy[i].posX);
            if(abs(i) = abs(diff)) then
            	break;
            end;
    until chksum = MAGIC;

if(_debug_) and (not mute) then
	writeLn('Checksum amounts to ', chksum);

if(not mute) then
	writeLn('Now checking for binary stars ...');

idx := 0;
	repeat
    clean := true;
    inc(idx);
    if(_debug_) and (not mute) then
    	writeLn('   pass #', idx);
    for i:=1 to 500 do
    	begin
        r := existStar(i, galaxy[i].posX, galaxy[i].posY);
        if(r>0) then
        	begin
        	inc(galaxy[i].posY, 2);
            if(not mute) then
            	writeLn(' Darkstar to earth : We exploded system ', i, ' + ', r);
            clean := false;
            end;
        end;
    until clean;

clean := test_hw_dist;
if((not clean) and (not mute)) then
	begin
    inc(frustration);
    if(frustration >= 10) then
		begin
        textColor(yellow);
        writeLn;
        writeLn('Minimum HW distance frustration:');
        writeLn(' I tried 10 times to move HWs to a minimum distance');
        writeLn(' of ', config.hw_mindist, 'LY. ');
        writeLn('You can now choose to reconfigure this parameter');
        writeLn('or let me try another 10 times:');
        writeLn;
        textColor(white);
        clean := not choice(' Shall I try again');
        writeLn;
        frustration := 0;
		end
    else
    	begin
    	writeLn;
    	writeLn('Minimum HW distances exceeded. I''l go again ...');
    	end;
    idx := 0;
    writeLn;
    end;

until clean;

if(not mute) then
	writeLn;
if(config.naming_policy = NAMES_SECRECY) then
	begin
    if(not mute) then
        writeLn('shuffling planet id''s to ensure secret naming policy ...');
    for i := 0 to 11 do
    	count[i] := 0;
    for i := 1 to 500 do
        swapPlanets(galaxy[i], galaxy[1+random(500)]);
    if(not mute) then
	    writeLn('Now naming each planet using the secret naming policy ...');
    for i := 1 to 500 do
		begin
        inc(count[galaxy[i].player]);
	    if(galaxy[i].player = 0) then
			galaxy[i].name := 'Vortex-' + strOfInt(count[galaxy[i].player])
        else
			galaxy[i].name := config.name_sector[galaxy[i].player] + '-' + strOfInt(count[galaxy[i].player]);
        end;
    end;
if(config.naming_policy = NAMES_EXPLICIT) then
	begin
    if(not mute) then
        writeLn('Now naming each planet using the explicit naming policy ...');
    for i := 1 to 11 do
    	begin
    	count[i] := 0;
        countm[i] := 0;
        end;
    count[0] := 0;
    for i := 1 to 500 do
    	begin
	        case galaxy[i].status of
    	    STAT_ANY:
        		begin
			    if(galaxy[i].player = 0) then
                	begin
                    inc(count[0]);
					galaxy[i].name := 'Vortex-' + strOfInt(count[0]);
                    end
      			else
                	begin
                    inc(count[galaxy[i].player]);
					galaxy[i].name := config.name_sector[galaxy[i].player] + '-' + strOfInt(count[galaxy[i].player]);
                    end;
    	        end;
        	STAT_MOON:
        		begin
	            inc(countm[galaxy[i].player]);
    	    	galaxy[i].name := config.name_sector[galaxy[i].player] + '-Moon-' + strOfInt(countm[galaxy[i].player]);
        	    end;
	        STAT_HW:
    	    	begin
        		galaxy[i].name := config.name_sector[galaxy[i].player] + '-Earth';
        		end;
        	end;
        end;
    end;

if(not mute) then
	begin
    textColor(green);
	writeLn;
	writeLn('Congratulations! You created a new cluster.');
	writeLn('You can now view it and if you like it compile the data files.');
	writeLn('If you don''t like it please re-run this generator.');
	writeLn;
    textColor(white);
	waitMessage('Press any key to continue');
	end;
end;

procedure view;

	procedure textout;
    var
     i, j		:integer;
     key1, key2 :char;
    begin
	cls(fdesk.colorset.bc);
	headline(lightgray, yellow, 'BIGBANG - visualization - text');
	statusline(lightgray, black, ' <+> to scroll down  <-> to scroll up  <ENTER> to abort');
    i := 0;
    	repeat
        gotoXY(1, 3);
        for j:=1 to 20 do
        	begin
            inc(i);
            if(i > 500) then
            	break;
            write('#', i, ' - ', galaxy[i].name, '  at (', galaxy[i].posX, '|', galaxy[i].posY, ')');
            if(_debug_) then
            	write('   stat=', galaxy[i].status, ' plid=', galaxy[i].player);
            clreol;
           	writeLn;
            end;
        repeat  getKeys(key1, key2) until key1 in['0', '+', '-', #13, #27];
        if(key1 in ['-']) then
            i := i - 40;
        if(key1 in ['0']) then
        	i := 0;
        if(i < 1) then
        	i := 1;
        if(i > 500) then
        	i := 500;
        until (key1 in[#27, #13]);
    statusline(lightgray, black, '');
    gotoXY(1, 23);
    waitMessage('Press any key to continue');
    end;

    procedure graphout(showall :boolean);
    var
     i, x, y, c	:integer;
     cmd		:char;
    begin
    if(not modeVGAHI(findBGIPATH)) then
    	begin
        textmode(CO80);
        writeLn;
        writeLn('ERROR: Graphics could not be initialized.');
		writeLn;
        writeLn('1. Please make sure the files egavga.bgi and litt.chr are in');
        writeLn('the current working directory or in the path specified in the');
        writeLn('parameter /BGI:<complete_bgipath>');
        writeLn('Your currently set BGI directory is :', findBGIPATH);
        writeLn;
        writeLn('2. This program uses the 640x480x16 VGA graphics mode');
        writeLn('Please check whether you have a VGA graphic card.');
        writeLn;
        waitMessage('Press any key to continue');
        exit;
        end;
    repeat
    graphCls(0);
	setColor(darkgray);
	circle(320, 240, round(config.r_vortex / 1000* 240 ));
	circle(320, 240, round((config.r_max-3)/ 1000 * 240));
	for i:=1 to config.num_sectors do
		line(320, 240,
		     round(320+config.r_max / 1000* 240 * cos(i*2*pi/config.num_sectors)),
		     round(240+config.r_max / 1000* 240 * sin(i*2*pi/config.num_sectors)));

	for i :=1 to 500 do
		begin
    	x := galaxy[i].posX;
    	y := galaxy[i].posY;
		putPixel(round((x-AVRX)/2000*480+320), round(240-(y-AVRY)/2000*480), white);
	    end;

    if(showall) then
    	begin
        readkey;
		circle(320, 240, round(config.r_hw / 1000* 240 ));
		for i :=1 to 500 do
			begin
    		x := galaxy[i].posX;
    		y := galaxy[i].posY;
            c := white;
           	case galaxy[i].status of
                STAT_HW: c := COL_HW;
                STAT_MOON: c := COL_MOON;
                STAT_ANY: c := COL_PLAYER[galaxy[i].player];
            end;
            putPixel(round((x-AVRX)/2000*480+320), round(240-(y-AVRY)/2000*480), c);
	   		end;
        end;

     cmd := readkey;
     if(cmd = 'r') then
     	create(true);
    until not (cmd in ['r']);
    closeGraph;
    textmode(CO80);
    end;

var
 i	:integer;
begin
	repeat
	cls(fdesk.colorset.bc);
	headline(lightgray, yellow, 'BIGBANG - visualization');
	statusline(lightgray, black, ' Colored graphics reveal HWs !');
	gotoXY(2, 3);
	i := menu('Display method', 'Text Graphics Colored'#255'graphics Quit'#255'menu' );

		case i of
    	1: textout;
	    2: graphout(false);
	    3: graphout(true)
		end;
	until i = 4;
end;

procedure saveUni;
var
 path	:string;
 f		:file of saveRecordType;
 i, idx	:integer;
 saveme	:saveRecordType;
begin
cls(fdesk.colorset.bc);
headline(lightgray, yellow, ' BIGBANG : save universe');
statusline(lightgray, black, '');
colors(blue, lightgray);
path := upStr(config.name_project+'.uni');
gotoXY(1, 3);
writeLn(' Saving this universe to .\', path, ' . ');
writeLn;
assign(f, path);
if(fileExist(path)) then
	begin
	if(_debug_) then
    	write(' Appending as ');
    reset(f);
    idx := filesize(f);
    seek(f, idx);
    if(_debug_) then
    	writeLn(idx+1, 'th record.');
    end
else
	begin
    if(_verbose_ or _debug_) then
    	writeLn(' Creating new file');
    rewrite(f);
    idx := 0;
    end;
saveme.versid := VERSION;
writeLn(' There are already ', idx, ' universes saved in this file.');
if(idx > MAXREC) then
	begin
    colors(blue, white);
    writeLn;
    writeLn('Sorry! No more than ', MAXREC, ' records allowed per project!');
    writeLn;
    waitMessage('Press any key to continue');
    exit;
    end;
writeLn(' You can now enter a short note to identify this universe.');
writeLn;
saveme.note := strInput(' Your comment', strOfInt(idx+1)+'th record', 60, true);
saveme.source := config;
for i:=1 to 500 do
	saveme.galaxy[i] := galaxy[i];
write(f, saveme);
close(f);
writeLn;
writeLn(' Succeeded in saving universe.');
writeLn;
waitMessage('Press any key to continue');
end;

procedure loadUni;
var
 name	:string[8];
 notes	:array[1..MAXREC] of string[60];
 path	:string;
 f		:file of saveRecordType;
 i, idx	:integer;
 recs	:integer;
 loadme	:saveRecordType;
 x, y	:integer;
begin
cls(fdesk.colorset.bc);
headline(lightgray, yellow, ' BIGBANG : load universe');
statusline(lightgray, black, '');
colors(blue, lightgray);
gotoXY(1, 3);
name := strInput(' Project name', config.name_project, 8, true);
if(name <> config.name_project) then
	begin
    writeLn;
    writeLn('You entered a project name that differs from the current project.');
    writeLn('The currently loaded configuration will be overwritten.');
    i := select('I want to', 'abort continue');
    	case i of
    	1: exit;
    	end;
	end;
writeLn;
path := upStr(name+'.uni');
if(not fileExist(path)) then
	begin
    writeLn(' The file ', path, ' doesn''t exist in the current directory!');
    writeLn;
    waitMessage('Press any key to continue');
    exit;
    end;
assign(f, path);
reset(f);
recs := filesize(f);
if(recs > MAXREC) then
	recs := MAXREC;
 for i:= 1 to recs do
	begin
    if(_debug_) then
    	writeLn(' loading record #', i);
    read(f, loadme);
    notes[i] := loadme.note;
    end;
close(f);

cls(fdesk.colorset.bc);
headline(lightgray, yellow, ' BIGBANG : load universe : select record');
statusline(lightgray, black, ' Choose a record 1..'+strOfInt(recs)+' or 0 to abort');
gotoXY(1, 3);
colors(fdesk.colorset.bc, white);
writeLn(' (0) Abort');
for i:= 1 to recs do
    writeLn(' ('+strOfInt(i)+') '+notes[i]);
writeLn;
write('Load record #');
x := getX; y := getY;
	repeat
    gotoXY(x, y);
    readLn(i);
    until(i in[0..recs]);
if(i = 0) then
	exit;
reset(f);
seek(f, i-1);
read(f, loadme);
config := loadme.source;
for idx:=1 to 500 do
	galaxy[idx] := loadme.galaxy[idx];
end;

procedure compile;

	procedure writeXYPLAN(g :galaxyType);
    var
     f	:file of integer;
     i, z	:integer;
    begin
    assign(f, 'xyplan.dat');
    rewrite(f);
    z := 0;
    for i:= 1 to 500 do
    	begin
        write(f, g[i].posX);
        write(f, g[i].posY);
        write(f, z);
        end;
    close(f);
    assign(f, 'xyplan.hst');
    rewrite(f);
    for i:= 1 to 500 do
    	begin
        write(f, g[i].posX);
        write(f, g[i].posY);
        if(g[i].status = STAT_HW) then
        	z := g[i].player
		else
        	z := 0;
        write(f, z);
        end;
    close(f);
    end;

    procedure writePLANETNM(g :galaxyType);
    var
     f	:file of char;
     i,j:integer;
     c	:char;
    begin
    c := ' ';
    assign(f, 'planet.nm');
    rewrite(f);
    for i:=1 to 500 do
        for j:=1 to 20 do
        	if(j <= length(g[i].name)) then
        		write(f, g[i].name[j])
            else
            	write(f, c);
    close(f);
    end;

    procedure writeHWID(g :galaxyType);
    var
     f	:text;
     i,j:integer;
    begin
    assign(f, 'hw.id');
    rewrite(f);
    writeLn(f, 'Recomended HW planets:');
    writeLn(f);
    for i:=1 to 500 do
    	if(g[i].status = STAT_HW) then
        	writeLn(f, '#', i, ' (', g[i].posX, '|', g[i].posY, ') is ', g[i].name);
    close(f);
    end;

var
 i	:integer;
begin
cls(fdesk.colorset.bc);
headline(lightgray, yellow, ' BIGBANG : datafile compiler');
statusline(lightgray, black, ' ');
colors(fdesk.colorset.bc, lightgray);
gotoXY(1, 3);
writeLn(' You are about to recompile the files');
writeLn('  ./xyplan.dat, ./xyplan.hst ./planet.nm .');
writeLn;
colors(fdesk.colorset.bc, white);
i := select('I want to', 'abort continue');
if(i = 1) then
	exit;
colors(fdesk.colorset.bc, lightgray);
writeLn;
writeLn('Writing XYPLANs');
writeXYPLAN(galaxy);
writeLn('Writing PLANETNM');
writePLANETNM(galaxy);
writeLn('Writing HW IDs');
writeHWID(galaxy);
writeLn;
writeLn(' Files recompiled. You must copy the files');
writeLn('  ./xyplan.dat ./xyplan.hst ./planet.nm to your host directory');
writeLn('  ./xyplan.dat ./planet.nm to your players directories.');
writeLn;
waitMessage('Press any key to continue');
end;

VAR
 i	:integer;

BEGIN
if(posParam('?') <> 0) then
	begin
    writeLn;
    writeLn('BIGBANG Copyright 1999 by Frederik Arbeiter');
    writeLn('Version : ', VERSION);
    writeLn;
    writeLn('Parameters');
    writeLn(' /BGI:<fullbgipath>    : the directory where egavga.bgi resides');
    writeLn(' /PN:<projectname>     : specify a projectname [8 characters]');
    writeLn(' /VERBOSE              : get a more detailed but messier output');
    writeLn(' /DEBUG			    : run in DUBUG_MODE. VERY messy output');
    writeLn;
    halt(0);
    end;

if(posParam('DEBUG') <> 0) then
	_debug_ := true
else
	_debug_ := false;
if(posParam('VERBOSE') <> 0) then
	_verbose_ := true
else
	_verbose_ := false;

if(not _debug_) then
	randomize;

defaults;
fdesk.colorset.bc := blue;
fdesk.colorset.tc := lightgray;
fdesk.colorset.hc := white;
	repeat
	cls(fdesk.colorset.bc);
	headLine(lightgray, yellow, ' BIGBANG v'+VERSION+' : main menu');
    if(_debug_) then
        statusLine(lightgray, black, ' Copyright 1999 by Frederik Arbeiter . Licence : GPL .          DEBUG_MODE = ON')
    else
        statusLine(lightgray, black, ' Copyright 1999 by Frederik Arbeiter . Licence : GPL');

	gotoXY(2, 3);
	i := menu('Main menu',
              'Configure'
			  +' Run'#255'universe'#255'generator'
			  +' View'#255'current'#255'universe'
			  +' Save'
			  +' Load'
			  +' Compile'#255'data'#255'files'
			  +' Exit'#255'program');
		case i of
		1:configure;
 		2:create(false);
 		3:view;
 		4:saveUni;
 		5:loadUni;
        6:compile;
		end;
	until i = 7;
cls(0);
writeLn;
writeLn('Thank you for using BIGBANG v', VERSION);
writeLn('Copyright 1999 by Frederik Arbeiter <frederik.arbeiter@gmx.net>');
writeLn('This program is free software. Please read the file COPYING for details');
writeLn;
END.